unit MabDlDlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons;

type
  TMabDeleteControl = (dcYesButton, dcNoButton);
  TMabDeleteDlg = class(TForm)
    Prompt: TLabel;
    imgDelete: TImage;
    YesBtn: TBitBtn;
    NoBtn: TBitBtn;
  private
    { Private declarations }
    FDescription: string;

    procedure SetButtonStyle(AButtonStyle: TButtonStyle);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;

    function Execute: Boolean;

    property ButtonStyle: TButtonStyle write SetButtonStyle;
    property Description: string read FDescription write FDescription;
  end;

   TMabDeleteDialog = class(TComponent)
   private
      { Private declarations }
      FActiveControl: TMabDeleteControl;
      FButtonStyle: TButtonStyle;
      FColor: TColor;
      FDescription: string;
      FFont: TFont;
      FTitle: string;
      FOnShow: TNotifyEvent;

      procedure SetFont(AFont: TFont);
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      function Execute: Boolean;
   published
      { Published declarations }
      property ActiveControl: TMabDeleteControl read FActiveControl write FActiveControl
         default dcYesButton;
      property ButtonStyle: TButtonStyle read FButtonStyle write FButtonStyle
         default bsAutoDetect;
      property Color: TColor read FColor write FColor;
      property Title: string read FTitle write FTitle;
      property Description: string read FDescription write FDescription;
      property Font: TFont read FFont write SetFont;

      property OnShow: TNotifyEvent read FOnShow write FOnShow;
   end;

var
  MabDeleteDlg: TMabDeleteDlg;

implementation

{$R *.DFM}

constructor TMabDeleteDlg.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FDescription := 'this item';
   if (Owner is TForm) then
   begin
      Font.Name := '';
      Font.Assign((Owner as TForm).Font);
   end;
end;

procedure TMabDeleteDlg.SetButtonStyle(AButtonStyle: TButtonStyle);
begin
   YesBtn.Style := AButtonStyle;
   NoBtn.Style := AButtonStyle;
end;

function TMabDeleteDlg.Execute: Boolean;
begin
   MessageBeep(MB_ICONQUESTION);
   Prompt.Caption := 'Are you sure you want to delete ' + Description + '?';
   Result := (ShowModal = mrYes);
end;

constructor TMabDeleteDialog.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FActiveControl := dcYesButton;
   FButtonStyle := bsAutoDetect;
   FColor := clBtnFace;
   FDescription := 'this item';
   FTitle := 'Confirm Delete';

   FFont := TFont.Create;
   FFont.Name := 'MS Sans Serif';
   FFont.Size := 8;
   FFont.Color := clWindowText;
   FFont.Style := [];

   FOnShow := nil;
end;

function TMabDeleteDialog.Execute: Boolean;
begin
   if Assigned(FOnShow) then FOnShow(Self);
   
   with TMabDeleteDlg.Create(Owner) do
   try
      Font.Name := ' ';
      Font.Assign(Self.Font);

      case Self.ActiveControl of
         dcYesButton: ActiveControl := YesBtn;
         dcNoButton:  ActiveControl := NoBtn;
      end;

      ButtonStyle := Self.ButtonStyle;
      Color := Self.Color;
      if Self.FTitle <> '' then Caption := Self.FTitle;
      if Self.FDescription <> '' then Description := Self.FDescription;

      Result := Execute;
   finally
      Free;
   end;
end;

procedure TMabDeleteDialog.SetFont(AFont: TFont);
begin
   if AFont <> FFont then
      FFont.Assign(AFont);
end;

end.
