unit MabImgCb;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TMabImageCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FImageList: TImageList;

    procedure SetImageList(Value: TImageList);
  protected
    { Protected declarations }
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer; var Height: Integer); override;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property Height;
    property HelpContext;
    property Hint;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property Items;
    property Left;
    property MaxLength;
    property Name;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Tag;
    property Text;
    property Top;
    property Visible;
    property Width;

    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;

    property ImageList: TImageList read FImageList write SetImageList;
  end;

implementation

constructor TMabImageCombo.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   Style := csOwnerDrawFixed;

   FImageList := nil;
end;

procedure TMabImageCombo.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = ImageList) then
      ImageList := nil;
end;

procedure TMabImageCombo.SetImageList(Value: TImageList);
begin
   if Value <> FImageList then
   begin
      FImageList := Value;

      if Value = nil then
         ItemHeight := Height
      else
      begin
         Height := FImageList.Height;
         ItemHeight := FImageList.Height;
      end;
   end;
end;

procedure TMabImageCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
   if Assigned(OnDrawItem) then
      OnDrawItem(Self, Index, Rect, State)
   else
   begin
      Canvas.FillRect(Rect);
      if Assigned(FImageList) then
      begin
         FImageList.Draw(Canvas, Rect.Left, Rect.Top, Index);
         Canvas.TextOut(Rect.Left + FImageList.Width + 2, Rect.Top + 1, Items[Index]);
      end
      else
         Canvas.TextOut(Rect.Left, Rect.Top + 1, Items[Index]);
   end;
end;

procedure TMabImageCombo.MeasureItem(Index: Integer; var Height: Integer);
begin
   if Assigned(FImageList) then Height := FImageList.Height;
end;

end.
