unit Mabrfsav;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MabRegSave, Registry;

type
   TFormProperty = (fpColor, fpFont, fpPosition, fpSize, fpWindowState);
   TFormProperties = set of TFormProperty;
   TMabRegFormSave = class(TMabRegSave)
   private
      { Private declarations }
      FRemember: TFormProperties;
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property Key;
      property Remember: TFormProperties read FRemember write FRemember;
   end;

implementation

constructor TMabRegFormSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [fpPosition, fpSize, fpWindowState];
end;

procedure TMabRegFormSave.Save;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if not (Owner is TForm) then
         raise Exception.Create('Owner must be of type TForm');

      with TForm(Owner) do
      begin
         if fpColor in Remember then
            if FRegistry.OpenKey(Key, True) then
            begin
               FRegistry.WriteString(ClassName, 'Color', ColorToString(Color));
               FRegistry.CloseKey;
            end;

         if fpFont in Remember then
         begin
            Section := 'Font';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               WriteFont(FRegistry, 'Font', Font);
               FRegistry.CloseKey;
            end;
            {
               FRegistry.WriteString (Section, 'Name',   Font.Name);
               FRegistry.WriteInteger(Section, 'Size',   Font.Size);
               FRegistry.WriteString (Section, 'Color',  ColorToString(Font.Color));
               FRegistry.WriteBool   (Section, 'Italic', (fsItalic in Font.Style));
               FRegistry.WriteBool   (Section, 'Bold',   (fsBold in Font.Style));
               FRegistry.WriteBool   (Section, 'Underline', (fsUnderline in Font.Style));
               FRegistry.WriteBool   (Section, 'Strikeout', (fsStrikeout in Font.Style));
            }
         end;

         if fpWindowState in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               if (WindowState = wsMinimized) or IsIconic(Application.Handle) then
                  FRegistry.WriteBool(Section, 'Minimized', True)

               else if WindowState = wsMaximized then
               begin
                  FRegistry.WriteBool(Section, 'Minimized', False);
                  FRegistry.WriteBool(Section, 'Maximized', True)
               end
               else if WindowState = wsNormal then
               begin
                  FRegistry.WriteBool(Section, 'Minimized', False);
                  FRegistry.WriteBool(Section, 'Maximized', False);
               end;
               FRegistry.CloseKey;
            end;
         end;

         WindowState := wsNormal;
         if fpPosition in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               FRegistry.WriteInteger(Section, 'Top', Top);
               FRegistry.WriteInteger(Section, 'Left', Left);
               FRegistry.CloseKey;
            end;
         end;

         if fpSize in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               FRegistry.WriteInteger(Section, 'Height', Height);
               FRegistry.WriteInteger(Section, 'Width', Width);
               FRegistry.CloseKey;
            end;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabRegFormSave.Load;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if not (Owner is TForm) then
         raise Exception.Create('Owner must be of type TForm');

      with TForm(Owner) do
      begin
         if fpColor in Remember then
            if FRegistry.OpenKey(Key, True) then
            begin
               Color := StringToColor(FRegistry.ReadString(ClassName, 'Color', ColorToString(Color)));
               FRegistry.CloseKey;
            end;

         if fpFont in Remember then
         begin
            Section := 'Font';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               ReadFont(FRegistry, 'Font', Font);
               FRegistry.CloseKey;
            end;
               {
               Font.Name  := FRegistry.ReadString(Section, 'Name', Font.Name);
               Font.Size  := FRegistry.ReadInteger(Section, 'Size', Font.Size);
               Font.Color := StringToColor(FRegistry.ReadString(Section, 'Color',
                          ColorToString(Font.Color)));

               AStyle := [];
               if FRegistry.ReadBool(Section, 'Italic', fsItalic in Font.Style) then
                  AStyle := AStyle + [fsItalic];

               if FRegistry.ReadBool(Section, 'Bold', fsBold in Font.Style) then
                  AStyle := AStyle + [fsBold];

               if FRegistry.ReadBool(Section, 'Underline', fsUnderline in Font.Style) then
                  AStyle := AStyle + [fsUnderline];

               if FRegistry.ReadBool(Section, 'Strikeout', fsStrikeout in Font.Style) then
                  AStyle := AStyle + [fsStrikeout];

               Font.Style := AStyle;
               }
         end;

         WindowState := wsNormal;
         if fpPosition in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               Top := FRegistry.ReadInteger(Section, 'Top', Top);
               Left := FRegistry.ReadInteger(Section, 'Left', Left);
               FRegistry.CloseKey;
            end;
         end;

         if fpSize in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               Height := FRegistry.ReadInteger(Section, 'Height', Height);
               Width := FRegistry.ReadInteger(Section, 'Width', Width);
               FRegistry.CloseKey;
            end;
         end;

         if fpWindowState in Remember then
         begin
            Section := 'Window';

            if FRegistry.OpenKey(Key + ClassName, True) then
            begin
               WindowState := wsNormal;
               if FRegistry.ReadBool(Section, 'Maximized', False) then
                  WindowState := wsMaximized;

               if FRegistry.ReadBool(Section, 'Minimized', False) then
                  WindowState := wsMinimized;
               FRegistry.CloseKey;
            end;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
