unit MabRTSav;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Forms, DBTables, MabRegSave, Registry;

type
   TMabTableProperty = (tpFieldDisplayWidths, tpFieldIndexes, tpFieldVisibles, tpFieldLabels, tpIndexFieldNames);
   TMabTableProperties = set of TMabTableProperty;
   TMabRegTableSave = class(TMabRegSave)
   private
      { Private declarations }
      FTable: TTable;
      FRemember: TMabTableProperties;
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Load;
      procedure Save;
   published
      { Published declarations }
      property Table: TTable read FTable write FTable;
      property Key;
      property Remember: TMabTableProperties read FRemember write FRemember;
   end;

implementation

constructor TMabRegTableSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [tpFieldDisplayWidths, tpFieldIndexes, tpFieldVisibles, tpIndexFieldNames]
end;

procedure TMabRegTableSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = Table) then
      Table := nil;
end;

procedure TMabRegTableSave.Load;
var
   AList: TStringList;
   PName, PMesg: PChar;
   i: Integer;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Table = nil then
         raise Exception.Create('Table must be specified');

      with Table do
      begin
         {
         We must use AList here, rather than TTable's
         FieldCount and Fields[] properties, since the
         index of each field may change
         }
         AList := TStringList.Create;
         try
            GetFieldNames(AList);
            for i := 0 to AList.Count - 1 do
               with FindField(AList.Strings[i]) do
                  if FRegistry.OpenKey(Key + Table.Owner.ClassName + '\' + Table.Name + '\Fields', True) then
                  begin
                     if tpFieldDisplayWidths in Remember then
                        DisplayWidth := FRegistry.ReadInteger(FieldName, 'DisplayWidth', DisplayWidth);
                     if tpFieldVisibles in Remember then
                        Visible := FRegistry.ReadBool(FieldName, 'Visible', Visible);
                     if tpFieldIndexes in Remember then
                        Index := FRegistry.ReadInteger(FieldName, 'Index', Index);
                     if tpFieldLabels in Remember then
                        DisplayLabel := FRegistry.ReadString(FieldName, 'DisplayLabel', DisplayLabel);

                     FRegistry.CloseKey;
                  end;
         finally
            AList.Free;
         end;

         if tpIndexFieldNames in Remember then
         begin
            if FRegistry.OpenKey(Key + Owner.ClassName, True) then
            begin
               IndexFieldNames := FRegistry.ReadString(Name, 'IndexFieldNames', '');
               FRegistry.CloseKey;
            end;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabRegTableSave.Save;
var
   PName, PMesg: PChar;
   i: Integer;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Table = nil then
         raise Exception.Create('Table must be specified');

      with Table do
      begin
         for i := 0 to FieldCount - 1 do
            with Fields[i] do
               if FRegistry.OpenKey(Key + Table.Owner.ClassName + '\' + Table.Name + '\Fields', True) then
               begin
                  if tpFieldDisplayWidths in Remember then
                     FRegistry.WriteInteger(FieldName, 'DisplayWidth', DisplayWidth);
                  if tpFieldIndexes in Remember then
                     FRegistry.WriteInteger(FieldName, 'Index', Index);
                  if tpFieldVisibles in Remember then
                     FRegistry.WriteBool(FieldName, 'Visible', Visible);
                  if tpFieldLabels in Remember then
                     FRegistry.WriteString(FieldName, 'DisplayLabel', DisplayLabel);
                  FRegistry.CloseKey;
               end;

         if tpIndexFieldNames in Remember then
         begin
            if FRegistry.OpenKey(Key + Owner.ClassName, True) then
            begin
               FRegistry.WriteString(Name, 'IndexFieldNames', IndexFieldNames);
               FRegistry.CloseKey;
            end;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
